{  --------------------------------------------------------------------------  }
{ SplashMs  -- Splash Message Component  Version 1   (c) AMS, Inc.  1997       }
{  --------------------------------------------------------------------------  }
{  Compatible with Delphi 1,2, and 3                                           }
{  --------------------------------------------------------------------------  }
{            Displays a Message above processing.                              }
{                                                                              }
{  Public Methods: Splash, Stop, Process                                       }
{  How it works:                                                               }
{  Start by calling the Splash method. Stop by calling the Stop method.        }
{  There is an OnCloseForm event attached to the OnClick event of all button   }
{  types except the help button.                                               }
{  Use the OnCloseForm event in your code to handle whatever                   }
{  processing you want to happen when the user clicks a button except help.    }
{  This component was designed to allow a user to discontinue some processing. }
{  Therefore, you can enter code in the OnCloseForm that will end a proccess   }
{  during application run-time.  Possibly by setting a flag.  Then call the    }
{  Stop method of this component to stop the message from displaying.          }
{  To have this component Act Modal.  Set the Modal property to true, then     }
{  Call Process in your code at the point of processing.  Setting Modal to     }
{  true will disable all components of the application, while calling the      }
{  Process method will allow Windows to process any events of the Splash       }
{  Message screen.                                                             }
{                                                                              }
{                  Notes about the some custom properties                      }
{                                                                              }
{  Canceled -- is a Public Property that is not visible on the Object Inspector}
{              but can be accessed in your application to check to see if the  }
{              Canceled button was clicked.                                    }
{                                                                              }
{  FixedWidth -- If the FixedWidth is set at a value greater than 0, then the  }
{                Splash Form will always be this width.                        }
{                                                                              }
{  Gauge -- The gauge property is activated when the ShowProgress property is  }
{           set to TRUE.                                                       }
{                                                                              }
{  MessageText -- The MessageText can be dynamically set at run-time.  This    }
{                 gives the ability to change the message as many times as     }
{                 is needed throughout the life of the component.              }
{                                                                              }
{  --------------------------------------------------------------------------  }
unit splashms;

interface

uses
  {$IFNDEF WIN32}
  WinProcs,     { Needed in D1 to use the DrawText method. }
  {$ENDIF}
  WinTypes, SysUtils, CommDlg, Messages, Classes, Graphics, Controls, Buttons,
  StdCtrls, ExtCtrls, Forms, Gauges,DsgnIntF,TypInfo, Gaugedit;
type

  TSplashGauge = Class( TPersistent ) { Create the gauge as a new class type }
  private
     Gauge       : TGauge; { Used/Created if ShowProgress is set to True }
     FBackColor,
     FForeColor,
     FColor      : TColor;
     FTop,
     FLeft,
     FHeight,
     FWidth,
     FMaxValue,
     FMinValue,
     FProgress   : Integer;
     FEnabled,
     FVisible    : boolean;
     Function GetColor: TColor;
     Function GetBackColor: TColor;
     Function GetForeColor: TColor;
     Function GetProgress: Integer;
     Procedure SetColor( Value : TColor );
     Procedure SetBackColor( Value : TColor );
     Procedure SetForeColor( Value : TColor );
     Procedure SetProgress( Value : Integer );
  public
     Property Enabled: boolean read FEnabled write FEnabled;
     Property Visible: boolean read FVisible write FVisible;
     Property Top:     Integer read FTop     write FTop;
     Property Left:    Integer read FLeft    write FLeft;
     property Height:  Integer read FHeight  write FHeight;
     property Width:   Integer read FWidth   write FWidth;
  published
     property Color    : TColor  read  GetColor     write SetColor;
     property BackColor: TColor  read  GEtBackColor write SetBackColor;
     property ForeColor: TColor  read  GetForeColor write SetForeColor;
     property MaxValue : Integer read  FMaxValue    write FMaxValue;
     property MinValue : Integer read  FMinValue    write FMinValue;
     property Progress : Integer read  GetProgress  write SetProgress;
  end;

  TSMsgDlgType = (smWarning, smError, smInformation, smConfirmation, smCustom);
  TSMsgDlgBtn = (smYes, smNo, smOK, smCancel, smAbort, smRetry, smIgnore, smAll, smHelp);
  TSMsgDlgButtons = set of TSMsgDlgBtn;

  TSplashMessage = class(TComponent)
  private
     { Private declarations }
     SplashForm   : TForm;
     FGauge       : TSplashGauge;
     FIcon        : TIcon;
     FPicture     : TPicture;
     FGaugeColor  : TColor;
     MsgLabel     : TLabel;
     FHelpContext : THelpContext;
     FBorderIcons : TBorderIcons;
     FBorderStyle : TFormBorderStyle;
     FDlgType     : TSMsgDlgType;
     FButtons     : TSMsgDlgButtons;
     FCloseForm   : TNotifyEvent;
     FFont        : TFont;
     FCaption     : String;
     FMessage     : String;
     FHint        : string;
     ButtonWasClicked : boolean;
     FCanceled    : boolean;
     SysClose     : boolean;
     NewIcon      : boolean;
     ReDraw       : boolean;
     FModal       : boolean;
     FEnabled     : boolean;
     FVisible     : boolean;
     FDefaultFont : boolean;
     FShowHint    : boolean;
     FShowProgress: boolean;
     OldLeft,
     OldTop       : Integer;
     FTop         : Integer;
     FHeight      : Integer;
     FWidth       : Integer;
     procedure SetPicture( Value : TPicture );
     function GetGauge: TSplashGauge;
     function Max(I, J: Integer): Integer;
     function GetAveCharSize(Canvas: TCanvas): TPoint;
     procedure SetGauge( Value : TSplashGauge );
     procedure SetButtonEvent( F : TForm );
     procedure EnableApplication;
     procedure FormClose(Sender: TObject; var Action: TCloseAction);
     procedure OnButtonClick( Sender : TObject );
     procedure SetFont( Value : TFont );
     procedure SetModal( Value : Boolean );
     procedure SetIcon( Value : TIcon );
     procedure SetWidth( Value : Integer );
     procedure SetMessage( Value : string );
     procedure SetDlgType( Value : TSMsgDlgType );
     function GetTextRect( aCanvas : TCanvas; Msg : string ): TRect;
     function CreateSplashMessage(const Msg: string; DlgType: TSMsgDlgType;
                                        AButtons: TSMsgDlgButtons;
                                        aGauge : TSplashGauge ): TForm;
  protected
    { Protected declarations }
  public
    { Public declarations }
    { This property does not appear on the Object inspector, but can be accssed }
    { Inside your application to check and see if a button was clicked. Any     }
    { button clicked, except the help button will close the Spashmessage form   }
    {    Therefore, the user can check to see if Canceled is True, and end stop }
    { any processing desired inside the application.                            }
    property Canceled:Boolean read FCanceled write FCanceled;
    procedure Splash;        { Call to Display the Splash Message Form }
    procedure Stop;          { Call to Stop the Splash Message Form }
    procedure Process;       { Call to allow Windows to process Windows Events }
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
  published
     { Published declarations }
     Property Gauge: TSplashGauge read GetGauge write SetGauge stored True;
     property BorderIcons: TBorderIcons read FBorderIcons write FBorderIcons;
     property BorderStyle: TFormBorderStyle read FBorderStyle write FBorderStyle;
     property Buttons: TSMsgDlgButtons read FButtons write FButtons;
     property Caption: string read FCaption write FCaption;
     property DefaultFont: boolean read FDefaultFont write FDefaultFont default true;
     property DialogType: TSMsgDlgType read FDlgType write SetDlgType;
     property Enabled: boolean read FEnabled write FEnabled default true;
     property Height: Integer read FHeight write FHeight;
     property HelpContext: THelpContext read FHelpContext write FHelpContext;
     property Hint: string read FHint write FHint;
     property Icon: TIcon read FIcon write SetIcon;
     property MessageText: string read FMessage write SetMessage;
     property Modal: boolean read FModal write SetModal default false;
     property Picture: TPicture read FPicture write SetPicture;
     property ScreenFont: TFont read FFont write SetFont;
     property ShowHint: boolean read FShowHint write FShowHint;
     property ShowProgress: boolean read FShowProgress write FShowProgress;
     property FixedWidth: Integer read FWidth write SetWidth;
     property Visible: boolean read FVisible write FVisible default true;
     { Custom CloseForm Event }
     property OnCloseForm: TNotifyEvent read FCloseForm write FCloseForm;
  end;
  
TGaugeProperty = class(TClassProperty)
  public
       function GetValue: string; override;
       function GetAttributes: TPropertyAttributes; override;
       procedure Edit; override;
  end;

var
   MWind : TSplashMessage;

procedure Register;

Implementation

const
  SOKButton = 61508;
  SCancelButton = 61509;
  SYesButton = 61510;
  SNoButton = 61511;
  SHelpButton = 61512;
  SCloseButton = 61513;
  SIgnoreButton = 61514;
  SRetryButton = 61515;
  SAbortButton = 61516;
  SAllButton = 61517;
  SMsgDlgWarning = 61560;
  SMsgDlgError = 61561;
  SMsgDlgInformation = 61562;
  SMsgDlgConfirm = 61563;
  SMsgDlgYes = 61564;
  SMsgDlgNo = 61565;
  SMsgDlgOK = 61566;
  SMsgDlgCancel = 61567;
  SMsgDlgHelp = 61568;
  SMsgDlgHelpNone = 61569;
  SMsgDlgHelpHelp = 61570;
  SMsgDlgAbort = 61571;
  SMsgDlgRetry = 61572;
  SMsgDlgIgnore = 61573;
  SMsgDlgAll = 61574;

  Captions: array[TSMsgDlgType] of Word = (SMsgDlgWarning, SMsgDlgError,
    SMsgDlgInformation, SMsgDlgConfirm, 0);
  MessageCaptions: array[TSMsgDlgType] of String = ('Warning','Error',
                'Information','Confirm','');
  IconIDs: array[TSMsgDlgType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
    IDI_ASTERISK, IDI_QUESTION, nil);
  ButtonNames: array[TSMsgDlgBtn] of string = (
    'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'Help');
  ButtonCaptions: array[TSMsgDlgBtn] of Word = (
    SMsgDlgYes, SMsgDlgNo, SMsgDlgOK, SMsgDlgCancel, SMsgDlgAbort,
    SMsgDlgRetry, SMsgDlgIgnore, SMsgDlgAll, SMsgDlgHelp);
  ModalResults: array[TSMsgDlgBtn] of Integer = (
    mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore, mrAll, 0);

type

  TMessageForm = class(TForm)
  private
    procedure HelpButtonClick(Sender: TObject);
  end;
{ HelpButtonClick - Sets the HelpContext String for the Help button }
procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;
{ ------------------ Gauge Functions and Procedures -------------------------- }
{ ---------------------------------------------------------------------------- }
Function TSplashGauge.GetColor: TColor;
begin
    If Gauge <> nil then
       Gauge.Color := FColor;
    Result := FColor;
end;
Procedure TSplashGauge.SetColor( Value : TColor );
begin
    If Gauge <> nil then
       Gauge.Color := Value;
    FColor := Value;
end;
Function TSplashGauge.GetBackColor: TColor;
begin
    If Gauge <> nil then
       Gauge.BackColor := FBAckColor;
    Result := FBackColor;
end;
Function TSplashGauge.GetForeColor: TColor;
begin
    If Gauge <> nil then
       Gauge.ForeColor := FForeColor;
    Result := FForeColor;
end;
Procedure TSplashGauge.SetBackColor(Value : TColor );
begin
    If Gauge <> nil then
       Gauge.BackColor := Value;
    FBackColor := Value;
end;
Procedure TSplashGauge.SetForeColor( Value : TColor );
begin
    If Gauge <> nil then
       Gauge.ForeColor := Value;
    FForeColor := Value;
end;

function TSplashGauge.GetProgress: Integer;
begin
    If Gauge <> nil then
       Gauge.Progress := FProgress;
    result := FProgress;
end;
procedure TSplashGauge.SetProgress( Value : Integer );
begin
       If Gauge <> nil then
          Gauge.Progress := Value;
       FProgress := Value;
end;
{ ---------------------------------------------------------------------------- }
{ ------------ Splash Message Functions and Procedures ----------------------- }
procedure TSplashMessage.SetModal( Value : Boolean );
begin
    FModal := Value;
    If FModal then if FButtons = [] then FButtons := [smCancel];
end;
procedure TSplashMessage.SetPicture( Value : TPicture );
begin
      FPicture.Assign( Value );
end;

procedure TSplashMessage.SetDlgType( Value : TSMsgDlgType );
begin
     FDlgType :=  Value;
     If ReDraw then
        NewIcon := True;
end;
{ SetButtonEvent - Sets up any buttons to the Custom OnButtonClick Event of    }
{                  this component.                                             }
procedure TSplashMessage.SetButtonEvent( F : TForm );
var
   I : Integer;
begin
     for I := 0 to F.ComponentCount -1 do
         if ( F.Components[I] is TButton ) And ( TButton(F.Components[I]).Name <> 'Help' ) Then
            TButton(F.Components[I]).OnClick := OnButtonClick;
end;

{ SetGauge -- Sets the TGauge prpoperties }
Function TSplashMessage.GetGauge: TSplashGauge;
begin
     Result := FGauge;
end;

procedure TSplashMessage.SetGauge( Value : TSplashGauge );
begin
       FGauge.Assign(Value);
end;
{ SetWidth - Allows the Width property to be Set at run-time                   }
procedure TSplashMessage.SetWidth( Value : Integer );
begin
    FWidth := Value;
    If (SplashForm <> nil ) then
       SplashForm.Width := FWidth;
end;
{ SetMessage - Allows the MessageText property to be changed at run-time       }
procedure TSplashMessage.SetMessage( Value : string );
begin
     FMessage := Value;
     If (SplashForm <> nil) Then
     begin
        ReDraw := True;
        MsgLabel.Caption := FMessage;
        If FWidth > 0 Then
        begin
           MsgLabel.BoundsRect := GetTextRect( SplashForm.Canvas, FMessage );
           MsgLabel.Left := OldLeft;
           MsgLabel.Top  := OldTop;
        end
        else
           SplashForm := CreateSplashMessage( FMessage, FDlgType, FButtons, FGauge );
        SplashForm.enabled := true;
     end;
end;

{ SetFont -- Sets the ScreenFont property                                     }
procedure TSplashMessage.SetFont( Value : TFont );
begin
     FFont.Assign( Value );
end;
{ SetIcon -- Sets the Icon property }
procedure TSplashMessage.SetIcon( Value : TIcon );
begin
     FIcon.Assign( Value );
end;
{ EnableApplication -- Enables all Forms belonging to the application }
procedure TSplashMessage.EnableApplication;
var i : integer;
begin
     If FModal then begin
         for i := 0 to screen.FormCount -1 do
         screen.Forms[I].enabled := true;
     end;
end;
{ Process -- Allows Windows to Process Window Events }
procedure TSplashMessage.Process;
begin
     If SplashForm <> nil then
        Application.ProcessMessages;
end;
{ Stop -- Stops displaying the Splash Message }
procedure TSplashMessage.Stop;
begin
     EnableApplication;           { EnableTheApplication if Modal is set on }
     If FGauge.Gauge <> nil then  { If a Progress Gauge is on the Form Then }
        FGauge.Gauge.Free;        { Free it .                               }
     If SplashForm <> nil Then       { If the Splash Form is not nil then      }
        SplashForm.Free;             { Free it.                                }
     FGauge.Gauge := nil;         { Set the Gauge point to nil              }
     SplashForm := nil;              { Set the SplashForm to point to nil      }
     ReDraw := false;             { Set the ReDraw flag to false.           }
end;

{  -------------------------------------------------------------------------  }
{ OnButtonClick Event -- Sets any buttons to the Custom OnformClose event     }
{                        Allows users to intrupt local processing before      }
{                        the Splash Message screen is closed.                 }
{  -------------------------------------------------------------------------  }
procedure TSplashMessage.OnButtonClick( sender : TObject );
begin
     FCanceled := True;
     EnableApplication;          { EnableTheApplication if Modal is set on }
     if assigned(FCloseForm) then FCloseForm(self);
     If SplashForm <> nil then
          SplashForm.Close;
     SplashForm := nil;
     ButtonWasClicked := True;
     ReDraw := false;
end;
{ FormClose Event -- The SplashMessage Custom OnFormClose Event }
procedure TSplashMessage.FormClose(Sender: TObject; var Action: TCloseAction);
begin

   If FGauge.Gauge <> nil then
   begin
     FGauge.Gauge.free;
     FGauge.Gauge := nil;
   end;
   Action := CaFree;
   If biSystemMenu in SplashForm.BorderIcons then
      SplashForm := nil;
end;

{  -------------------------------------------------------------------------  }
{  Splash -- Creates and Display's the Splash Message screen                  }
{  -------------------------------------------------------------------------  }
procedure TSplashMessage.Splash;
var i : integer;
begin
    FCanceled := false;
     { Don't allow multiple instances }
    If SplashForm = nil Then
    begin
         If FModal then
            for i := 0 to screen.FormCount -1 do
                screen.Forms[I].enabled := false;

       ReDraw := False;

       SplashForm := CreateSplashMessage( FMessage, FDlgType, FButtons, FGauge );

       With SplashForm do
       begin    { Reset the OnClick Event of all Buttons that belong to this form }
           HelpContext := 0;
           ScaleBy(Screen.PixelsPerInch, 96);

           Enabled := FEnabled;
           { Height, and Top are set in CreateMessageDialog. However, you can
           { override their settings by inserting a value greater than 0.  }
           If FHeight > 0 then
              Height := FHeight;
           If FTop > 0 then
              Top := FTop;
           Hint := FHint;
           BorderIcons := FBorderIcons;
           BorderStyle := FBorderStyle;
           If FCaption <> '' Then
              Caption := FCaption;
           If Not FDefaultFont Then begin
              Font.Name  := FFont.Name;
              Font.Style := FFont.Style;
              Font.Size  := FFont.Size;
              Font.Color := FFont.Color;
           end;
           ShowHint := FShowHint;
           HelpContext := FHelpContext;
           Icon := FIcon;
           Visible := FVisible;
           OnClose := FormClose;
           SetButtonEvent(SplashForm);
           Show;
       end;
       end
       else
           SplashForm.Show;
       Application.ProcessMessages;  { Allows the application to process window events }
end;
{ GetTextRect - Sets the Width of the Message. This is called when the Splash  }
{               message is created, and also when the message is changed.      }
function TSplashMessage.GetTextRect( aCanvas : TCanvas; Msg : string ): TRect;
var
    C: array[0..255] of Char;
begin
    Result := Rect(0, 0, Screen.Width div 2, 0);
    DrawText(aCanvas.Handle, StrPCopy(C, Msg), -1, Result, DT_CALCRECT or
      DT_WORDBREAK);
end;
{ Max - Used to Set The ClientWidth property of the Splash Message Form during }
{       the initial create process.                                            }
function TSplashMessage.Max(I, J: Integer): Integer;
begin
  if I > J then Result := I else Result := J;
end;
{  --------------------------------------------------------------------------  }
{ GetAveCharSize - Gets the Average size of a Character Byte for the Splash    }
{                  Form's Canvas property.  This code is taken from the 32-bit }
{                  dialog.pas file. The API function GetTextExtentPoint is     }
{                  used to calculate the ave height and width of the currently }
{                  selected font.                                              }
{  --------------------------------------------------------------------------  }
function TSplashMessage.GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;
{  --------------------------------------------------------------------------  }
{ CreateSplashMessage - Is where the SplashMessage Form is created. This is    }
{                       called when the form is initially created, and also    }
{                       when the form's message has been changed ( unless the  }
{                       fixed width is set to True ).                          }
{                       the bulk of this code is taken from the 32-bit version }
{                       of dialog.pas and has been modified for this component }
{  --------------------------------------------------------------------------  }
function TSplashMessage.CreateSplashMessage(const Msg: string; DlgType: TSMsgDlgType;
  AButtons: TSMsgDlgButtons; aGauge : TSplashGauge ): TForm;
const
  mcHorzMargin = 8;
  mcVertMargin = 8;
  mcHorzSpacing = 10;
  mcVertSpacing = 10;
  mcButtonWidth = 50;
  mcButtonHeight = 14;
  mcButtonSpacing = 4;
  ProgressBarTopAdjustment = 25;
var
  I,J : Integer;
  N   : array[0..100] of Byte;
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
  ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
  IconTextWidth, IconTextHeight, X: Integer;
  B, DefaultButton, CancelButton: TSMsgDlgBtn;
  IconID: PChar;
  TextRect: TRect;
begin
  If ReDraw Then
      Result := SplashForm
  Else
      Result := TForm.CreateNew(self);
  with Result do
  begin
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
    TextRect := GetTextRect(Canvas, Msg );
    IconID := IconIDs[DlgType];
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    If ( IconId <> nil ) Or ( Picture.Graphic <> nil ) then
    begin
         Inc(IconTextWidth, 32 + HorzSpacing);
         if IconTextHeight < 32 then IconTextHeight := 32;
    end;
    ButtonCount := 0;
    for B := Low(TSMsgDlgBtn) to High(TSMsgDlgBtn) do
      if B in Buttons then Inc(ButtonCount);
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    If FWidth > 0 Then
        ClientWidth := FWidth
    else
        ClientWidth := Max(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;

    If FShowProgress then
        ClientHeight := IconTextHeight + ButtonHeight + VertSpacing + VertMargin * 3
    else
        ClientHeight := IconTextHeight + ButtonHeight + VertSpacing + VertMargin * 2;
    { do not re-position if re-drawing the form }
    If NOt ReDraw then
    begin
         Left := (Screen.Width div 2) - (Width div 2);
         Top := (Screen.Height div 2) - (Height div 2);
    end;
    if DlgType <> smCustom then
      Caption := MessageCaptions[DlgType] else
      Caption := Application.Title;

   If Not ReDraw Or NewIcon then { Do Not create the Icon again if re-drawing }
   begin
        If NewIcon then
        begin
              J:=0;
              for I := 0 to ComponentCount -1 do
                 if ( Components[I] is TImage ) Then begin
                     N[J] := I;
                     Inc(J);
                 end;
              for I := 0 to J-1 do
                    Components[N[I]].free;
              NewIcon := false;
         end;

        if (IconID <> nil) Or (Picture <> nil) then
        with TImage.Create(Result) do
        begin
             Parent := Result;
             Stretch := True;
             If (FDlgType = (smCustom)) and (Picture <> (Nil) )then
                 Picture := FPicture
             else
                 Picture.Icon.Handle := LoadIcon(0, IconID);
             SetBounds(HorzMargin, VertMargin, 32, 32);
        end;
    end;

    if Not ReDraw then  { Do Not re-create TLabel if re-drawing }
       MsgLabel := TLabel.Create(Result);
    with MsgLabel as TLabel do
    begin
      Name := 'Message';
      Parent := Result;
      WordWrap := True;
      Caption := Msg;
      BoundsRect := TextRect;
      SetBounds(IconTextWidth - TextRect.Right + HorzMargin, VertMargin,
          TextRect.Right, TextRect.Bottom);
      OldLeft := Left;
      OldTop  := Top;
      Refresh;
    end;

    If FShowProgress Then
    begin
         If Not ReDraw then
            FGauge.Gauge := TGauge.Create(result);

         with FGauge.Gauge as TGauge do
         begin
              Parent := Result;
              Width  := Result.Width - ( Result.Width div 5 );
              Left   := (( Result.Width div 2) - ( Width div 2 ))-2;
              If ( IconId = nil ) and ( Picture.Graphic = nil ) then
                  Top := MsgLabel.Top + ProgressBarTopAdjustment
              else
                  Top    := MsgLabel.Top + 38;
              Height := FGauge.FHeight;
              Visible := True;
              Enabled := True;
              MaxValue := FGauge.MaxValue;
              MinValue := FGauge.MinValue;
              Color := FGauge.Color;
              BackColor := FGauge.BackColor;
              ForeColor := FGauge.ForeColor;
         end;
    end;

    if smOk in Buttons then DefaultButton := smOk else
      if smYes in Buttons then DefaultButton := smYes else
        DefaultButton := smRetry;
    if smCancel in Buttons then CancelButton := smCancel else
      if smNo in Buttons then CancelButton := smNo else
        CancelButton := smOk;
    X := (ClientWidth - ButtonGroupWidth) div 2;

   If ReDraw then
   begin   { Centerize any Buttons components on the Form }
      for I := 0 to ComponentCount -1 do
               if ( Components[I] is TButton ) Then
                      if FShowProgress then
                      TButton(Components[I]).SetBounds(X, IconTextHeight +
                      VertMargin + VertSpacing + (ProgressBarTopAdjustment-5),
                      ButtonWidth, ButtonHeight)
                      else
                      TButton(Components[I]).SetBounds(X, IconTextHeight +
                      VertMargin + VertSpacing,
                      ButtonWidth, ButtonHeight);

   end
   else

       for B := Low(TSMsgDlgBtn) to High(TSMsgDlgBtn) do
       begin
            if B in Buttons then
            begin
                 with TButton.Create(Result) do
                 begin
                      Name := ButtonNames[B];
                      Parent := Result;
                      {Caption := LoadStr(ButtonCaptions[B]);}
                      Caption := Name;
                      if B = DefaultButton then Default := True;
                      if B = CancelButton then Cancel := True;
                      if FShowProgress then
                      SetBounds(X, IconTextHeight + VertMargin + VertSpacing +
                      (ProgressBarTopAdjustment-5),
                      ButtonWidth, ButtonHeight)
                      else
                      SetBounds(X, IconTextHeight + VertMargin + VertSpacing,
                      ButtonWidth, ButtonHeight);
                      Inc(X, ButtonWidth + ButtonSpacing);
                      if B = smHelp then
                      OnClick := TMessageForm(Result).HelpButtonClick;
                 end;
            end;
       end;
   end;
end;

{  -------------------------------------------------------------------------  }
{  Create Constructor -- Creates this Component.                              }
{  -------------------------------------------------------------------------  }
Constructor TSplashMessage.Create( AOwner : TComponent );
begin
     inherited Create( AOwner );
     FFont := TFont.Create;
     With FFont As TFont do
     begin
          FFont.Name := 'Arial';
          FFont.Size := 8;
          FFont.Style := [];
          FFont.Color := clBlack;
     end;
     FIcon := TIcon.Create;
     { Set Defaults }
     FPicture := TPicture.Create;
     FMessage := 'Splash Message Component';
     FDlgType := (smInformation);
     FButtons := [smCancel];
     FBorderIcons := [biSystemMenu];
     FDefaultFont := True;
     FBorderStyle := (bsDialog);
     SysClose := false;
     FEnabled := True;
     FVisible := True;
     ReDraw := False;
     ButtonWasClicked := false;
     FGauge := TSplashGauge.Create;
     with FGauge as TSplashGauge do
     begin
           FHeight := 20;
           FGaugeColor := clBtnFace;
           Color := FGaugeColor;
           FBackColor := clWhite;
           BAckColor := FBackColor;
           FForeColor := clActiveCaption;
           ForeColor := FForeColor;
           FMaxValue := 100;
     end;
 end;
{  -------------------------------------------------------------------------  }
{  Destroy Destructor -- Destroy's the Splash Message Component.              }
{  -------------------------------------------------------------------------  }
Destructor TSplashMessage.Destroy;
begin
    If FGauge.Gauge <> nil then begin
       FGauge.Gauge.free;
       FGauge.Gauge := nil;
    end;
    If FGauge <> nil Then begin
       FGauge.free;
       FGauge := nil;
    end;
    FFont.Free;
    FIcon.Free;
    FPicture.free;
    SplashForm.Free;
    SplashForm := nil;
   { If MWind <> nil Then begin}
        MWind.Free;
        MWind := nil;
   { end;}

    inherited Destroy;
end;
function TGaugeProperty.GetValue: string;
begin
     FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;
function TGAugeProperty.GetAttributes: TPropertyAttributes;
begin
      Result := [paMultiSelect,paDialog,paSubProperties];
end; { GetAttributes }
procedure TGaugeProperty.Edit;
begin
      with TForm_GaugeEdit.Create(nil) do { Create the Gauge Property Editor Form }
      try
         { Copy the Property Values From the Object Inspector to the Property Editor Form }
         SpinEdit_Max.Value := TSplashGauge(GetOrdValue).MaxValue;
         SpinEdit_Min.Value := TSplashGauge(GetOrdValue).MinValue;
         SpinEdit_Prog.Value := TSplashGauge(GetOrdValue).Progress;
         Sh_Color.Brush.Color := TSplashGauge(GetOrdValue).Color;
         Sh_BackColor.Brush.Color := TSplashGauge(GetOrdValue).BackColor;
         Sh_ForeColor.Brush.Color := TSplashGauge(GetOrdValue).ForeColor;
         If ShowModal = mrOk then begin
            { Copy the Property Editor Form values to the Object Inspector's values }
            TSplashGauge(GetOrdValue).MaxValue  := SpinEdit_Max.Value;
            TSplashGauge(GetOrdValue).MinValue  := SpinEdit_Min.Value;
            TSplashGauge(GetOrdValue).Progress  := SpinEdit_Prog.Value;
            TSplashGauge(GetOrdValue).Color     := Sh_Color.Brush.Color;
            TSplashGauge(GetOrdValue).BackColor := Sh_BackColor.Brush.Color;
            TSplashGauge(GetOrdValue).ForeColor := Sh_ForeColor.Brush.Color;  
         end;
      finally
         free; { Free the Gauge Property Editor Form }
      end;
  { The Designer property provides a read-only reference to the               }
  { form-designer object containing the property or component being edited.   }
  { Property and component editors call the Modified method whenever anything }
  { in a component changes, allowing the designer to reflect the change.      }
      Designer.Modified;
end; { Edit }
{  -------------------------------------------------------------------------  }
{  Register -- Registers this component, and place in the Samples Tab, of the }
{              Delphi Component Palette.                                      }
{  -------------------------------------------------------------------------  }
procedure Register;
begin
     RegisterComponents('Samples', [TSplashMessage]);
     RegisterPropertyEditor(TypeInfo(TSplashGauge),nil,'',TGaugeProperty)
end;

end.
